@@ -99,6 +99,7 @@ M_OBJECTS = "Within objects"
 M_IMAGES_AND_OBJECTS = "Both"
 
 M_FAST = "Fast"
+M_FASTER = "Faster"
 M_ACCURATE = "Accurate"
 
 """Feature name format for the correlation measurement"""
@@ -249,16 +250,22 @@ Select *{YES}* to run the Manders coefficients using Costes auto threshold.
 
         self.fast_costes = Choice(
             "Method for Costes thresholding",
-            [M_FAST, M_ACCURATE],
+            [M_FASTER, M_FAST, M_ACCURATE],
             doc=f"""\
 This setting determines the method used to calculate the threshold for use within the
-Costes calculations. Selecting *{M_FAST}* will prioritise testing the most relevant potential
-thresholds to identify the optimal value. Selecting *{M_ACCURATE}* will test every possible
-threshold value. The latter method becomes particularly time-consuming with larger images.
-In most instances the results of both strategies should be identical.
+Costes calculations. The *{M_FAST}* and *{M_ACCURATE}* modes will test candidate thresholds
+in descending order until the optimal threshold is reached. Selecting *{M_FAST}* will attempt 
+to skip candidates when results are far from the optimal value being sought. Selecting *{M_ACCURATE}* 
+will test every possible threshold value. When working with 16-bit images these methods can be extremely 
+time-consuming. Selecting *{M_FASTER}* will use a modified bisection algorithm to find the threshold 
+using a shrinking window of candidates. This is substantially faster but may produce slightly lower 
+thresholds in exceptional circumstances.
 
-In both modes Costes automatic thresholding can seriously impact performance when working with 16-bit images.
-You may want to disable these specific measurements (available when "*Run All Metrics?*" is set to "*No*").
+In the vast majority of instances the results of all strategies should be identical. We recommend using 
+*{M_FAST}* mode when working with 8-bit images and *{M_FASTER}* mode when using 16-bit images.
+
+Alternatively, you may want to disable these specific measurements entirely 
+(available when "*Run All Metrics?*" is set to "*No*").
 """
         )
 
@@ -515,65 +522,18 @@ You may want to disable these specific measurements (available when "*Run All Me
 
             if self.do_costes:
                 # Orthogonal Regression for Costes' automated threshold
-                nonZero = (fi > 0) | (si > 0)
+                scale = get_scale(first_image.scale, second_image.scale)
+                if self.fast_costes == M_FASTER:
+                    thr_fi_c, thr_si_c = self.bisection_costes(fi, si, scale)
+                else:
+                    thr_fi_c, thr_si_c = self.linear_costes(fi, si, scale)
 
-                xvar = numpy.var(fi[nonZero], axis=0, ddof=1)
-                yvar = numpy.var(si[nonZero], axis=0, ddof=1)
-
-                xmean = numpy.mean(fi[nonZero], axis=0)
-                ymean = numpy.mean(si[nonZero], axis=0)
-
-                z = fi[nonZero] + si[nonZero]
-                zvar = numpy.var(z, axis=0, ddof=1)
-
-                covar = 0.5 * (zvar - (xvar + yvar))
-
-                denom = 2 * covar
-                num = (yvar - xvar) + numpy.sqrt(
-                    (yvar - xvar) * (yvar - xvar) + 4 * (covar * covar)
-                )
-                a = num / denom
-                b = ymean - a * xmean
-
-                i_step = get_scale(first_image.scale, second_image.scale)
-
-                # Start at 1 step above the maximum
-                img_max = max(fi.max(), si.max())
-                i = i_step * ((img_max // i_step) + 1)
-
-                num_true = None
-                while i > i_step:
-                    Thr_fi_c = i
-                    Thr_si_c = (a * i) + b
-                    combt = (fi < Thr_fi_c) | (si < Thr_si_c)
-                    try:
-                        # Only run pearsonr if the input has changed.
-                        if (positives := numpy.count_nonzero(combt)) != num_true:
-                            costReg, _ = scipy.stats.pearsonr(fi[combt], si[combt])
-                            num_true = positives
-                        if costReg <= 0:
-                            break
-                        elif self.fast_costes.value == M_ACCURATE or i < i_step * 10:
-                            i -= i_step
-                        elif costReg > 0.45:
-                            # We're way off, step down 10x
-                            i -= i_step * 10
-                        elif costReg > 0.35:
-                            # Still far from 0, step 5x
-                            i -= i_step * 5
-                        elif costReg > 0.25:
-                            # Step 2x
-                            i -= i_step * 2
-                        else:
-                            i -= i_step
-                    except ValueError:
-                        break
                 # Costes' thershold calculation
-                combined_thresh_c = (fi > Thr_fi_c) & (si > Thr_si_c)
+                combined_thresh_c = (fi > thr_fi_c) & (si > thr_si_c)
                 fi_thresh_c = fi[combined_thresh_c]
                 si_thresh_c = si[combined_thresh_c]
-                tot_fi_thr_c = fi[(fi > Thr_fi_c)].sum()
-                tot_si_thr_c = si[(si > Thr_si_c)].sum()
+                tot_fi_thr_c = fi[(fi > thr_fi_c)].sum()
+                tot_si_thr_c = si[(si > thr_si_c)].sum()
 
                 # Costes' Automated Threshold
                 C1 = 0
@@ -1080,57 +1040,14 @@ You may want to disable these specific measurements (available when "*Run All Me
                 ]
 
             if self.do_costes:
-                nonZero = (fi > 0) | (si > 0)
-                xvar = numpy.var(fi[nonZero], axis=0, ddof=1)
-                yvar = numpy.var(si[nonZero], axis=0, ddof=1)
+                # Orthogonal Regression for Costes' automated threshold
+                scale = get_scale(first_image.scale, second_image.scale)
 
-                xmean = numpy.mean(fi[nonZero], axis=0)
-                ymean = numpy.mean(si[nonZero], axis=0)
+                if self.fast_costes == M_FASTER:
+                    thr_fi_c, thr_si_c = self.bisection_costes(fi, si, scale)
+                else:
+                    thr_fi_c, thr_si_c = self.linear_costes(fi, si, scale)
 
-                z = fi[nonZero] + si[nonZero]
-                zvar = numpy.var(z, axis=0, ddof=1)
-
-                covar = 0.5 * (zvar - (xvar + yvar))
-
-                denom = 2 * covar
-                num = (yvar - xvar) + numpy.sqrt(
-                    (yvar - xvar) * (yvar - xvar) + 4 * (covar * covar)
-                )
-                a = num / denom
-                b = ymean - a * xmean
-
-                i_step = get_scale(first_image.scale, second_image.scale)
-
-                # Start at 1 step above the maximum
-                img_max = max(fi.max(), si.max())
-                i = i_step * ((img_max // i_step) + 1)
-                num_true = None
-                while i > i_step:
-                    thr_fi_c = i
-                    thr_si_c = (a * i) + b
-                    combt = (fi < thr_fi_c) | (si < thr_si_c)
-                    try:
-                        # Only run pearsonr if the input has changed.
-                        if (positives := numpy.count_nonzero(combt)) != num_true:
-                            costReg, _ = scipy.stats.pearsonr(fi[combt], si[combt])
-                            num_true = positives
-                        if costReg <= 0:
-                            break
-                        elif self.fast_costes.value == M_ACCURATE or i < i_step * 10:
-                            i -= i_step
-                        elif costReg > 0.45:
-                            # We're way off, step down 10x
-                            i -= i_step * 10
-                        elif costReg > 0.35:
-                            # Still far from 0, step 5x
-                            i -= i_step * 5
-                        elif costReg > 0.25:
-                            # Step 2x
-                            i -= i_step * 2
-                        else:
-                            i -= i_step
-                    except ValueError:
-                        break
                 # Costes' thershold for entire image is applied to each object
                 fi_above_thr = first_pixels > thr_fi_c
                 si_above_thr = second_pixels > thr_si_c
@@ -1317,6 +1234,142 @@ You may want to disable these specific measurements (available when "*Run All Me
         else:
             return result
 
+    def linear_costes(self, fi, si, scale_max=255):
+        """
+        Finds the Costes Automatic Threshold for colocalization using a linear algorithm.
+        Candiate thresholds are gradually decreased until Pearson R falls below 0.
+        If "Fast" mode is enabled the "steps" between tested thresholds will be increased
+        when Pearson R is much greater than 0.
+        """
+        i_step = 1 / scale_max
+        non_zero = (fi > 0) | (si > 0)
+        xvar = numpy.var(fi[non_zero], axis=0, ddof=1)
+        yvar = numpy.var(si[non_zero], axis=0, ddof=1)
+
+        xmean = numpy.mean(fi[non_zero], axis=0)
+        ymean = numpy.mean(si[non_zero], axis=0)
+
+        z = fi[non_zero] + si[non_zero]
+        zvar = numpy.var(z, axis=0, ddof=1)
+
+        covar = 0.5 * (zvar - (xvar + yvar))
+
+        denom = 2 * covar
+        num = (yvar - xvar) + numpy.sqrt(
+            (yvar - xvar) * (yvar - xvar) + 4 * (covar * covar)
+        )
+        a = num / denom
+        b = ymean - a * xmean
+
+        # Start at 1 step above the maximum value
+        img_max = max(fi.max(), si.max())
+        i = i_step * ((img_max // i_step) + 1)
+
+        num_true = None
+        fi_max = fi.max()
+        si_max = si.max()
+
+        # Initialise without a threshold
+        costReg, _ = scipy.stats.pearsonr(fi, si)
+        thr_fi_c = i
+        thr_si_c = (a * i) + b
+        while i > fi_max and (a * i) + b > si_max:
+            i -= i_step
+        while i > i_step:
+            thr_fi_c = i
+            thr_si_c = (a * i) + b
+            combt = (fi < thr_fi_c) | (si < thr_si_c)
+            try:
+                # Only run pearsonr if the input has changed.
+                if (positives := numpy.count_nonzero(combt)) != num_true:
+                    costReg, _ = scipy.stats.pearsonr(fi[combt], si[combt])
+                    num_true = positives
+
+                if costReg <= 0:
+                    break
+                elif self.fast_costes.value == M_ACCURATE or i < i_step * 10:
+                    i -= i_step
+                elif costReg > 0.45:
+                    # We're way off, step down 10x
+                    i -= i_step * 10
+                elif costReg > 0.35:
+                    # Still far from 0, step 5x
+                    i -= i_step * 5
+                elif costReg > 0.25:
+                    # Step 2x
+                    i -= i_step * 2
+                else:
+                    i -= i_step
+            except ValueError:
+                break
+        return thr_fi_c, thr_si_c
+
+    def bisection_costes(self, fi, si, scale_max=255):
+        """
+        Finds the Costes Automatic Threshold for colocalization using a bisection algorithm.
+        Candidate thresholds are selected from within a window of possible intensities,
+        this window is narrowed based on the R value of each tested candidate.
+        We're looking for the first point below 0, and R value can become highly variable
+        at lower thresholds in some samples. Therefore the candidate tested in each
+        loop is 1/6th of the window size below the maximum value (as opposed to the midpoint).
+        """
+
+        non_zero = (fi > 0) | (si > 0)
+        xvar = numpy.var(fi[non_zero], axis=0, ddof=1)
+        yvar = numpy.var(si[non_zero], axis=0, ddof=1)
+
+        xmean = numpy.mean(fi[non_zero], axis=0)
+        ymean = numpy.mean(si[non_zero], axis=0)
+
+        z = fi[non_zero] + si[non_zero]
+        zvar = numpy.var(z, axis=0, ddof=1)
+
+        covar = 0.5 * (zvar - (xvar + yvar))
+
+        denom = 2 * covar
+        num = (yvar - xvar) + numpy.sqrt(
+            (yvar - xvar) * (yvar - xvar) + 4 * (covar * covar)
+        )
+        a = num / denom
+        b = ymean - a * xmean
+
+        # Initialise variables
+        left = 1
+        right = scale_max
+        mid = ((right - left) // (6/5)) + left
+        lastmid = 0
+        # Marks the value with the last positive R value.
+        valid = 1
+
+        while lastmid != mid:
+            thr_fi_c = mid / scale_max
+            thr_si_c = (a * thr_fi_c) + b
+            combt = (fi < thr_fi_c) | (si < thr_si_c)
+            if numpy.count_nonzero(combt) <= 2:
+                # Can't run pearson with only 2 values.
+                left = mid - 1
+            else:
+                try:
+                    costReg, _ = scipy.stats.pearsonr(fi[combt], si[combt])
+                    if costReg < 0:
+                        left = mid - 1
+                    elif costReg >= 0:
+                        right = mid + 1
+                        valid = mid
+                except ValueError:
+                    # Catch misc Pearson errors with low sample numbers
+                    left = mid - 1
+            lastmid = mid
+            if right - left > 6:
+                mid = ((right - left) // (6 / 5)) + left
+            else:
+                mid = ((right - left) // 2) + left
+
+        thr_fi_c = (valid - 1) / scale_max
+        thr_si_c = (a * thr_fi_c) + b
+
+        return thr_fi_c, thr_si_c
+
     def get_measurement_columns(self, pipeline):
         """Return column definitions for all measurements made by this module"""
         columns = []
@@ -1562,10 +1615,10 @@ You may want to disable these specific measurements (available when "*Run All Me
 
 def get_scale(scale_1, scale_2):
     if scale_1 is not None and scale_2 is not None:
-        return 1 / max(scale_1, scale_2)
+        return max(scale_1, scale_2)
     elif scale_1 is not None:
-        return 1 / scale_1
+        return scale_1
     elif scale_2 is not None:
-        return 1 / scale_2
+        return scale_2
     else:
-        return 1 / 255
+        return 255
